/**
 * SAMPLE CODE NOTICE
 * 
 * THIS SAMPLE CODE IS MADE AVAILABLE AS IS.  MICROSOFT MAKES NO WARRANTIES, WHETHER EXPRESS OR IMPLIED,
 * OF FITNESS FOR A PARTICULAR PURPOSE, OF ACCURACY OR COMPLETENESS OF RESPONSES, OF RESULTS, OR CONDITIONS OF MERCHANTABILITY.
 * THE ENTIRE RISK OF THE USE OR THE RESULTS FROM THE USE OF THIS SAMPLE CODE REMAINS WITH THE USER.
 * NO TECHNICAL SUPPORT IS PROVIDED.  YOU MAY NOT DISTRIBUTE THIS CODE UNLESS YOU HAVE A LICENSE AGREEMENT WITH MICROSOFT THAT ALLOWS YOU TO DO SO.
 */

import StoreCardDiscountOperationResponse from "./StoreCardDiscountOperationResponse";
import StoreCardDiscountOperationRequest from "./StoreCardDiscountOperationRequest";
import { ExtensionOperationRequestHandlerBase, ExtensionOperationRequestType } from "PosApi/Create/Operations";
import { ClientEntities } from "PosApi/Entities";
import * as CartOperations from "PosApi/Consume/Cart";

/**
 * (Sample) Request handler for the StoreCardDiscountOperationRequest class.
 */

export default class StoreCardDiscountOperationRequestHandler<TResponse extends StoreCardDiscountOperationResponse> extends ExtensionOperationRequestHandlerBase<TResponse> {
    
    /**
     * Gets the supported request type.
     * @return {RequestType<TResponse>} The supported request type.
     */
    public supportedRequestType(): ExtensionOperationRequestType<TResponse> {
        return StoreCardDiscountOperationRequest;
    }

    /**
     * Executes the request handler asynchronously.
     * @param {StoreCardDiscountOperationRequest<TResponse>} request The request.
     * @return {Promise<ICancelableDataResult<TResponse>>} The cancelable async result containing the response.
     */
    public executeAsync(request: StoreCardDiscountOperationRequest<TResponse>): Promise<ClientEntities.ICancelableDataResult<TResponse>> {

        let addAffiliationRequest: CartOperations.AddAffiliationOperationRequest<CartOperations.AddAffiliationOperationResponse> =
            new CartOperations.AddAffiliationOperationRequest(["StoreCreditCardDiscountAffiliation",], [], request.correlationId);

        return this.context.runtime.executeAsync(addAffiliationRequest)
            .then((
                result: ClientEntities.ICancelableDataResult<CartOperations.AddAffiliationOperationResponse>
            ): ClientEntities.ICancelableDataResult<TResponse> => {

                return <ClientEntities.ICancelableDataResult<TResponse>>{
                    canceled: result.canceled,
                    data: new StoreCardDiscountOperationResponse()
                };
            });
    }
}